/******************************************************************************
* FILE NAME       : "VU meter" test code
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <math.h>

#ifdef WIN32
#include <windows.h>
#include <process.h>
#include <io.h>
#define usleep(a) Sleep((a)/1000)
#else
#include <unistd.h>
#include <sys/time.h>
#endif

#ifdef WIN32
#  define THREAD HANDLE
#else
#  include <pthread.h>
#  define THREAD pthread_t
#endif

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Slider.H>

#include "audio_sample.h"

#define QUSED	((audio_data.qin - audio_data.qout) & QMASK)

#define SAMPLE_BUF_LEN 2048
static short sample[SAMPLE_BUF_LEN];
static short active_tasks = 0;
static int level = 0;
int exitTest = 0;

Fl_Double_Window *outer_win=(Fl_Double_Window *)0;
Fl_Slider *level_slide=(Fl_Slider *)0;

/******************************************************************************
*	make_thread
******************************************************************************/
THREAD make_thread (void (*task)(void *), void *arg)
{
	int res;
#ifdef WIN32
    res = _beginthread(task, 0, arg);
	return (THREAD)res;
#else
	pthread_t thread;
	res = pthread_create(&thread, NULL, (void*(*)(void*))task, arg);
//	res = pthread_create(&thread, NULL, task, arg);
	if (res == 0)
		return thread;
	else
		return (THREAD)0;
#endif
}
/******************************************************************************
*	cb_Exit
******************************************************************************/
static void cb_Exit(Fl_Return_Button*, void*)
{
	exitTest = 99;
	outer_win->hide();
}

/******************************************************************************
*	find_peak
******************************************************************************/
static void find_peak (short * data)
{
	long i;
	short max = 0;
	static float vu = 0.0;

	for (i = 0; i < SAMPLE_BUF_LEN; i++)
	{
		if (abs(data[i]) > max) max = abs(data[i]);
	}
	vu = (vu * 0.6) + ((float)max * 0.4);

	level = (int)vu;
} /* end of find_peak */

/******************************************************************************
*   draw_wave
******************************************************************************/
void draw_wave(void* p)
{
	unsigned long qidx, i;

	int status = start_audio();
	if (status != 0) exitTest = 1;

	while (exitTest == 0)
	{
		if (QUSED > SAMPLE_BUF_LEN)
		{
			qidx = (audio_data.qin - SAMPLE_BUF_LEN) & QMASK;

			// read sample data from audio queue into complex array
			for (i = 0; i < SAMPLE_BUF_LEN; i++)
			{
				sample[i] = audio_data.qbuf[qidx]; // convert to float
				qidx = (qidx + 1) & QMASK;
			}
			audio_data.qout = audio_data.qin; // empty the queue buffer

			/* do something to the data... */
			find_peak(sample);
		}
		// sleep a bit until it's time to try again
		if (exitTest == 0) usleep (5000);
	} // end of main display processing loop
	status = stop_audio();
	active_tasks --;
}

/******************************************************************************
*   thread callbacks
******************************************************************************/
void wave_CB (void *)
{
	static int old_level = 0;
	if (level != old_level)
	{
		old_level = level;
		level_slide->value((double)old_level);
		level_slide->redraw();
	}
	if (exitTest == 0)
		Fl::repeat_timeout(0.1, (Fl_Timeout_Handler)wave_CB);
}
/*****************************************************************************/
void kill_threads ( void)
{
	exitTest = 99;
}
/******************************************************************************
*   main
******************************************************************************/
int main (int argc, char *argv[])
{
    Fl::visual(FL_RGB);
    Fl::visual(FL_DOUBLE|FL_INDEX);
//    Fl::scheme("plastic");

	outer_win = new Fl_Double_Window(221, 404, "Volume Display");
	outer_win->when(FL_WHEN_NEVER);
	outer_win->begin();
	{
		Fl_Return_Button* o = new Fl_Return_Button(130, 360, 75, 30, "Exit");
		o->tooltip("Exit");
		o->labeltype(FL_ENGRAVED_LABEL);
		o->labelcolor((Fl_Color)4);
		o->callback((Fl_Callback*)cb_Exit);
	}

	level_slide = new Fl_Slider(65, 20, 45, 315, "LEVEL");
	level_slide->selection_color((Fl_Color)1);
	level_slide->minimum(32768);
	level_slide->maximum(0);
	level_slide->type(FL_VERT_FILL_SLIDER);
	level_slide->when(FL_WHEN_NEVER);

	outer_win->end();

// init the memory strucures
	audio_data.qin = audio_data.qout = 0;

// Start the display refresh timeout
	Fl::add_timeout(0.2, (Fl_Timeout_Handler)wave_CB);

// Start the subsidiary tasks
	make_thread (draw_wave, NULL);
	active_tasks ++;

	// Run the GUI - when this returns, it's time to exit
	outer_win->show(argc, argv);
	Fl::run();

	/* Now tell the sub-threads to terminate */
	kill_threads();
	while (active_tasks > 0) // wait for all the sub-threads to terminate
	{
		usleep (100);
	}
	return 0;
}
/* END OF FILE */
